/*
    SPDX-License-Identifier: GPL-3.0

    Copyright (C) 2020-2025  Jevgenijs Protopopovs

    This file is part of Kefir project.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KEFIR_TARGET_ASM_AMD64_DB_H_
#define KEFIR_TARGET_ASM_AMD64_DB_H_

#include "kefir/core/basic-types.h"

#define KEFIR_AMD64_INSTRDB_FLAG(_index) (1ull << (_index))

#define KEFIR_AMD64_INSTRDB_NONE 0ull
// Operand flags
// Access flags
#define KEFIR_AMD64_INSTRDB_READ KEFIR_AMD64_INSTRDB_FLAG(0)
#define KEFIR_AMD64_INSTRDB_WRITE KEFIR_AMD64_INSTRDB_FLAG(1)
// General purpose operand flags
#define KEFIR_AMD64_INSTRDB_GP_REGISTER8 KEFIR_AMD64_INSTRDB_FLAG(8)
#define KEFIR_AMD64_INSTRDB_GP_REGISTER16 KEFIR_AMD64_INSTRDB_FLAG(9)
#define KEFIR_AMD64_INSTRDB_GP_REGISTER32 KEFIR_AMD64_INSTRDB_FLAG(10)
#define KEFIR_AMD64_INSTRDB_GP_REGISTER64 KEFIR_AMD64_INSTRDB_FLAG(11)
// SSE operand flags
#define KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE KEFIR_AMD64_INSTRDB_FLAG(12)
#define KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE KEFIR_AMD64_INSTRDB_FLAG(13)
#define KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL KEFIR_AMD64_INSTRDB_FLAG(14)
// Memory operand flags
#define KEFIR_AMD64_INSTRDB_MEMORY8 KEFIR_AMD64_INSTRDB_FLAG(15)
#define KEFIR_AMD64_INSTRDB_MEMORY16 KEFIR_AMD64_INSTRDB_FLAG(16)
#define KEFIR_AMD64_INSTRDB_MEMORY32 KEFIR_AMD64_INSTRDB_FLAG(17)
#define KEFIR_AMD64_INSTRDB_MEMORY64 KEFIR_AMD64_INSTRDB_FLAG(18)
#define KEFIR_AMD64_INSTRDB_MEMORY_XMM KEFIR_AMD64_INSTRDB_FLAG(19)
#define KEFIR_AMD64_INSTRDB_MEMORY_FPU_SINGLE KEFIR_AMD64_INSTRDB_FLAG(20)
#define KEFIR_AMD64_INSTRDB_MEMORY_FPU_DOUBLE KEFIR_AMD64_INSTRDB_FLAG(21)
#define KEFIR_AMD64_INSTRDB_MEMORY_FPU_LONG_DOUBLE KEFIR_AMD64_INSTRDB_FLAG(22)
// x87 fpu flags
#define KEFIR_AMD64_INSTRDB_FPU_STACK KEFIR_AMD64_INSTRDB_FLAG(23)
// Other flags
#define KEFIR_AMD64_INSTRDB_LABEL KEFIR_AMD64_INSTRDB_FLAG(24)
#define KEFIR_AMD64_INSTRDB_IMMEDIATE KEFIR_AMD64_INSTRDB_FLAG(25)
// Instruction flags
#define KEFIR_AMD64_INSTRDB_BOTH_MEMORY KEFIR_AMD64_INSTRDB_FLAG(26)

// Flag combinations
#define KEFIR_AMD64_INSTRDB_READ_WRITE (KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_WRITE)
#define KEFIR_AMD64_INSTRDB_GP_REGISTER                                                                         \
    (KEFIR_AMD64_INSTRDB_GP_REGISTER8 | KEFIR_AMD64_INSTRDB_GP_REGISTER16 | KEFIR_AMD64_INSTRDB_GP_REGISTER32 | \
     KEFIR_AMD64_INSTRDB_GP_REGISTER64)
#define KEFIR_AMD64_INSTRDB_GP_MEMORY                                                            \
    (KEFIR_AMD64_INSTRDB_MEMORY8 | KEFIR_AMD64_INSTRDB_MEMORY16 | KEFIR_AMD64_INSTRDB_MEMORY32 | \
     KEFIR_AMD64_INSTRDB_MEMORY64)
#define KEFIR_AMD64_INSTRDB_FPU_MEMORY                                               \
    (KEFIR_AMD64_INSTRDB_MEMORY_FPU_SINGLE | KEFIR_AMD64_INSTRDB_MEMORY_FPU_DOUBLE | \
     KEFIR_AMD64_INSTRDB_MEMORY_FPU_LONG_DOUBLE)
#define KEFIR_AMD64_INSTRDB_ANY_MEMORY \
    (KEFIR_AMD64_INSTRDB_GP_MEMORY | KEFIR_AMD64_INSTRDB_MEMORY_XMM | KEFIR_AMD64_INSTRDB_FPU_MEMORY)
#define KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY (KEFIR_AMD64_INSTRDB_GP_REGISTER | KEFIR_AMD64_INSTRDB_GP_MEMORY)
#define KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE \
    (KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE | KEFIR_AMD64_INSTRDB_MEMORY32)
#define KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE \
    (KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE | KEFIR_AMD64_INSTRDB_MEMORY64)
#define KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL \
    (KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL | KEFIR_AMD64_INSTRDB_MEMORY_XMM)
#define KEFIR_AMD64_INSTRDB_XMM_REGISTER_ANY                                             \
    (KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE | \
     KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL)
#define KEFIR_AMD64_INSTRDB_FPU_STACK_MEMORY (KEFIR_AMD64_INSTRDB_FPU_STACK | KEFIR_AMD64_INSTRDB_FPU_MEMORY)

#define KEFIR_AMD64_INSTRDB_BRANCH_TAGET                                                           \
    (KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER64 | KEFIR_AMD64_INSTRDB_MEMORY64 | \
     KEFIR_AMD64_INSTRDB_LABEL)
#define KEFIR_AMD64_INSTRDB_SETCC_TARGET \
    (KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER8 | KEFIR_AMD64_INSTRDB_MEMORY8)

// clang-format off
#define KEFIR_AMD64_INSTRUCTION_DATABASE(_instr0, _instr1, _instr2, _instr3, _separator) \
    /* Control flow */ \
    _instr1(call, "call", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jmp, "jmp", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jz, "jz", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jnz, "jnz", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(je, "je", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jo, "jo", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jne, "jne", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(js, "js", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jns, "jns", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jp, "jp", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jnp, "jnp", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(ja, "ja", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jae, "jae", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jb, "jb", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jnb, "jnb", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jbe, "jbe", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jg, "jg", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jge, "jge", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jl, "jl", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr1(jle, "jle", BRANCH, KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_BRANCH_TAGET) _separator \
    _instr0(ret, "ret", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(ud2, "ud2", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    /* Data moves */ \
    _instr1(push, "push", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER) _separator \
    _instr1(pop, "pop", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER) _separator \
    _instr2(mov, "mov", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(xchg, "xchg", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr2(movabs, "movabs", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(movzx, "movzx", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(movsx, "movsx", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmovl, "cmovl", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmove, "cmove", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmovbe, "cmovbe", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmovb, "cmovb", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmovne, "cmovne", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmovs, "cmovs", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmovns, "cmovns", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(lea, "lea", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_LABEL) _separator \
    _instr0(movsb, "movsb", REPEATABLE, KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(movsw, "movsw", REPEATABLE, KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(movsl, "movsl", REPEATABLE, KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(movsq, "movsq", REPEATABLE, KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(stosb, "stosb", REPEATABLE, KEFIR_AMD64_INSTRDB_NONE) _separator \
    /* Flags */ \
    _instr0(pushfq, "pushfq", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(popfq, "popfq", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(cld, "cld", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    /* Conditionals */ \
    _instr2(test, "test", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(btc, "btc", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmp, "cmp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr1(sete, "sete", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setne, "setne", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setnp, "setnp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setp, "setp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setg, "setg", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setge, "setge", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setl, "setl", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setle, "setle", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(seta, "seta", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setae, "setae", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setb, "setb", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setnb, "setnb", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setbe, "setbe", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(seto, "seto", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(sets, "sets", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    _instr1(setc, "setc", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_SETCC_TARGET) _separator \
    /* Integer arithmetics */ \
    _instr2(add, "add", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(adc, "adc", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(sub, "sub", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(sbb, "sbb", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr1(mul, "mul", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr2(imul, "imul", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr3(imul3, "imul", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr1(idiv, "idiv", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr1(div, "div", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr2(shl, "shl", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER8 | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(shr, "shr", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER8 | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(sar, "sar", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER8 | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr0(cwd, "cwd", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(cdq, "cdq", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(cqo, "cqo", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr2(and, "and", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(or, "or", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(xor, "xor", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr1(not, "not", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr1(neg, "neg", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr1(dec, "dec", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr2(bsf, "bsf", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr2(bsr, "bsr", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr2(rol, "rol", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER | KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr1(bswap, "bswap", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER32 | KEFIR_AMD64_INSTRDB_GP_REGISTER64) _separator \
    /* SSE */ \
    _instr2(movd, "movd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(movq, "movq", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(movd2, "movd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER32 | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE) _separator \
    _instr2(movq2, "movq", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER64 | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE) _separator \
    _instr2(movdqu, "movdqu", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(movaps, "movaps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(unpcklpd, "unpcklpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(unpckhpd, "unpckhpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(cvtsi2ss, "cvtsi2ss", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr2(cvtsi2sd, "cvtsi2sd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr2(cvttss2si, "cvttss2si", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(cvttsd2si, "cvttsd2si", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(cvtss2sd, "cvtss2sd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(cvtsd2ss, "cvtsd2ss", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(cvtps2pd, "cvtps2pd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(cvtpd2ps, "cvtpd2ps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(pxor, "pxor", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(addss, "addss", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(addps, "addps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(addsd, "addsd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(addpd, "addpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(subss, "subss", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(subsd, "subsd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(mulss, "mulss", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(mulps, "mulps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(mulsd, "mulsd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(mulpd, "mulpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(divss, "divss", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(divsd, "divsd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(divpd, "divpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL) _separator \
    _instr2(xorps, "xorps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(xorpd, "xorpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(ucomiss, "ucomiss", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(ucomisd, "ucomisd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(comiss, "comiss", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(comisd, "comisd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr1(stmxcsr, "stmxcsr", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_MEMORY32) _separator \
    _instr1(ldmxcsr, "ldmxcsr", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_MEMORY32) _separator \
    _instr3(pextrq, "pextrq", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr3(pinsrq, "pinsrq", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY, \
        KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr3(shufps, "shufps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL, \
        KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr3(shufpd, "shufpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_FULL, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_FULL, \
        KEFIR_AMD64_INSTRDB_IMMEDIATE) _separator \
    _instr2(cmpeqps, "cmpeqps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(cmpeqpd, "cmpeqpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(cmpneqps, "cmpneqps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(cmpneqpd, "cmpneqpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(andps, "andps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(andpd, "andpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    _instr2(orps, "orps", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_SINGLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_SINGLE) _separator \
    _instr2(orpd, "orpd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ_WRITE | KEFIR_AMD64_INSTRDB_XMM_REGISTER_DOUBLE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_XMM_REGISTER_MEMORY_DOUBLE) _separator \
    /* x87 */ \
    _instr1(fld, "fld", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_FPU_STACK_MEMORY) _separator \
    _instr1(fild, "fild", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_MEMORY16 | KEFIR_AMD64_INSTRDB_MEMORY32 | KEFIR_AMD64_INSTRDB_MEMORY64) _separator \
    _instr0(fldz, "fldz", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(fld1, "fld1", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr1(fistp, "fistp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_MEMORY16 | KEFIR_AMD64_INSTRDB_MEMORY32 | KEFIR_AMD64_INSTRDB_MEMORY64) _separator \
    _instr1(fstp, "fstp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_FPU_STACK_MEMORY) _separator \
    _instr0(fabs, "fabs", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr1(fadd, "fadd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_MEMORY_FPU_SINGLE | KEFIR_AMD64_INSTRDB_MEMORY_FPU_DOUBLE) _separator \
    _instr2(fadd2, "fadd", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr0(faddp, "faddp", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr1(faddp1, "faddp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fsubp2, "fsubp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fsubrp2, "fsubrp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr0(fsubp, "fsubp", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr1(fsubp1, "fsubp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr0(fmulp, "fmulp", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr1(fmul1, "fmul", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fmul2, "fmul", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fmulp2, "fmulp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr1(fmulp1, "fmulp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fdiv2, "fdiv", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fdivr2, "fdivr", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr0(fdivp, "fdivp", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr1(fdivp1, "fdivp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fdivp2, "fdivp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fdivrp2, "fdivrp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr1(fdivrp1, "fdivrp", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr0(fchs, "fchs", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr1(fxch, "fxch", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr1(fucomi, "fucomi", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr1(fucomip, "fucomip", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr1(fcomip, "fcomip", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr2(fcomip2, "fcomip", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK, KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr1(fcomi, "fcomi", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_FPU_STACK) _separator \
    _instr1(fstcw, "fstcw", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_MEMORY16) _separator \
    _instr1(fldcw, "fldcw", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_READ | KEFIR_AMD64_INSTRDB_MEMORY16) _separator \
    /* fenv */ \
    _instr1(fnstenv, "fnstenv", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_MEMORY) _separator \
    _instr1(fnstcw, "fnstcw", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr1(fnstsw, "fnstsw", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_WRITE | KEFIR_AMD64_INSTRDB_GP_REGISTER_MEMORY) _separator \
    _instr0(fnclex, "fnclex", , KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr1(fldenv, "fldenv", , KEFIR_AMD64_INSTRDB_NONE, \
        KEFIR_AMD64_INSTRDB_GP_MEMORY) _separator \
    /* Prefixes */ \
    _instr0(lock, "lock", PREFIX, KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(rep, "rep", PREFIX, KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(data16, "data16", PREFIX, KEFIR_AMD64_INSTRDB_NONE) _separator \
    _instr0(rexW, "rex.W", PREFIX, KEFIR_AMD64_INSTRDB_NONE)
// clang-format on

#endif
