/*
    SPDX-License-Identifier: GPL-3.0

    Copyright (C) 2020-2025  Jevgenijs Protopopovs

    This file is part of Kefir project.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "kefir/ast-translator/translator_impl.h"
#include "kefir/ast-translator/translator.h"
#include "kefir/ast-translator/value.h"
#include "kefir/ast-translator/lvalue.h"
#include "kefir/ast/type_conv.h"
#include "kefir/core/util.h"
#include "kefir/core/error.h"

kefir_result_t kefir_ast_translate_array_subscript_node(struct kefir_mem *mem,
                                                        struct kefir_ast_translator_context *context,
                                                        struct kefir_irbuilder_block *builder,
                                                        const struct kefir_ast_array_subscript *node) {
    REQUIRE(mem != NULL, KEFIR_SET_ERROR(KEFIR_INVALID_PARAMETER, "Expected valid memory allocator"));
    REQUIRE(context != NULL, KEFIR_SET_ERROR(KEFIR_INVALID_PARAMETER, "Expected valid AST translation context"));
    REQUIRE(builder != NULL, KEFIR_SET_ERROR(KEFIR_INVALID_PARAMETER, "Expected valid IR block builder"));
    REQUIRE(node != NULL, KEFIR_SET_ERROR(KEFIR_INVALID_PARAMETER, "Expected valid AST array subscript node"));

    kefir_bool_t atomic_aggregate_element = false;
    REQUIRE_OK(kefir_ast_translate_array_subscript_lvalue(mem, context, builder, node));
    if (node->base.properties.expression_props.atomic) {
        REQUIRE_OK(kefir_ast_translator_atomic_load_value(node->base.properties.type, context->ast_context->type_traits,
                                                          builder, &atomic_aggregate_element));
    } else {
        REQUIRE_OK(
            kefir_ast_translator_load_value(node->base.properties.type, context->ast_context->type_traits, builder));
    }

    if (atomic_aggregate_element) {
        REQUIRE_OK(kefir_ast_translator_load_atomic_aggregate_value(
            mem, node->base.properties.type, context, builder,
            &node->base.properties.expression_props.temporary_identifier, &node->base.source_location));
    }
    return KEFIR_OK;
}
