KEFIR_END2END_BIN_PATH=$(KEFIR_BIN_DIR)/tests/end2end
KEFIR_END2END_TEST_LIBS=-pthread

ifeq ($(PLATFORM),linux)
ifneq ($(KEFIR_TEST_USE_MUSL),yes)
KEFIR_END2END_TEST_LIBS+=-latomic
endif
endif
ifeq ($(PLATFORM),freebsd)
KEFIR_END2END_TEST_LIBS+=-lstdthreads -lcompiler_rt
endif
ifeq ($(PLATFORM),openbsd)
KEFIR_END2END_TEST_LIBS+=-lcompiler_rt
endif
ifeq ($(PLATFORM),netbsd)
KEFIR_END2END_TEST_LIBS+=-latomic
endif

$(KEFIR_END2END_BIN_PATH)/%.o: CFLAGS += -Wno-int-in-bool-context -UNDEBUG

$(KEFIR_END2END_BIN_PATH)/%.kefir.o: $(SOURCE_DIR)/tests/end2end/%.kefir.c $(KEFIR_BIN_DIR)/kefir
	@mkdir -p $(shell dirname "$@")
	@echo "Kefir-Compile $@"
	@AS="$(AS)" \
	 VALGRIND_TEST_OPTIONS="$(VALGRIND_TEST_OPTIONS)" \
	 USE_VALGRIND="$(USE_VALGRIND)" \
	 PLATFORM="$(PLATFORM)" \
	 $(SOURCE_DIR)/tests/end2end/compile.sh $(KEFIR_BIN_DIR) $< $@

$(KEFIR_END2END_BIN_PATH)/%.test: KEFIR_END2END_TEST_KEFIR_BIN_DIR=$(KEFIR_END2END_BIN_PATH)/$(patsubst %.test,%,$(notdir $@))
$(KEFIR_END2END_BIN_PATH)/%.test: KEFIR_END2END_TEST_SOURCE_DIR=$(KEFIR_END2END_TEST_KEFIR_BIN_DIR:$(KEFIR_BIN_DIR)/%=$(SOURCE_DIR)/%)
$(KEFIR_END2END_BIN_PATH)/%.test: KEFIR_END2END_TEST_HOST_SOURCE=$(wildcard $(KEFIR_END2END_TEST_SOURCE_DIR)/*.host.c)
$(KEFIR_END2END_BIN_PATH)/%.test: KEFIR_END2END_TEST_KEFIR_SOURCE=$(wildcard $(KEFIR_END2END_TEST_SOURCE_DIR)/*.kefir.c)
$(KEFIR_END2END_BIN_PATH)/%.test: KEFIR_END2END_TEST_HOST_OBJECT_FILES=$(KEFIR_END2END_TEST_HOST_SOURCE:$(SOURCE_DIR)/%.c=$(KEFIR_BIN_DIR)/%.o)
$(KEFIR_END2END_BIN_PATH)/%.test: KEFIR_END2END_TEST_KEFIR_OBJECT_FILES=$(KEFIR_END2END_TEST_KEFIR_SOURCE:$(SOURCE_DIR)/%.c=$(KEFIR_BIN_DIR)/%.o)

.SECONDEXPANSION:
$(KEFIR_END2END_BIN_PATH)/%.test: $$(KEFIR_END2END_TEST_HOST_OBJECT_FILES) $$(KEFIR_END2END_TEST_KEFIR_OBJECT_FILES)
	@mkdir -p $(shell dirname "$@")
	@echo "Linking $@"
	@$(CC) $(TEST_CFLAGS) $(SANITIZER_FLAGS) $(KEFIR_END2END_TEST_HOST_OBJECT_FILES) $(KEFIR_END2END_TEST_KEFIR_OBJECT_FILES) -o $@ $(KEFIR_END2END_TEST_LIBS) -lm $(LDFLAGS)

$(KEFIR_END2END_BIN_PATH)/%.test.done: $(KEFIR_END2END_BIN_PATH)/%.test
	@echo "Running $<"
ifeq ($(USE_VALGRIND),yes)
	@valgrind $(VALGRIND_TEST_OPTIONS) $<
else
	$<
endif
	@touch $@

$(KEFIR_END2END_BIN_PATH)/%.asmgen.output: $(SOURCE_DIR)/tests/end2end/%.kefir.asmgen.c $(KEFIR_BIN_DIR)/kefir
	@mkdir -p $(shell dirname "$@")
	@echo "Kefir-Translate $@"
	@ASMGEN=yes \
	 AS="$(AS)" \
	 VALGRIND_TEST_OPTIONS="$(VALGRIND_TEST_OPTIONS)" \
	 USE_VALGRIND="$(USE_VALGRIND)" \
	 PLATFORM="$(PLATFORM)" \
	 	$(SOURCE_DIR)/tests/end2end/compile.sh $(KEFIR_BIN_DIR) $< $@

$(KEFIR_END2END_BIN_PATH)/%.also.asmgen.output: $(SOURCE_DIR)/tests/end2end/%.kefir.c $(KEFIR_BIN_DIR)/kefir
	@mkdir -p $(shell dirname "$@")
	@echo "Kefir-Translate $@"
	@ASMGEN=yes \
	 AS="$(AS)" \
	 VALGRIND_TEST_OPTIONS="$(VALGRIND_TEST_OPTIONS)" \
	 USE_VALGRIND="$(USE_VALGRIND)" \
	 PLATFORM="$(PLATFORM)" \
	 	$(SOURCE_DIR)/tests/end2end/compile.sh $(KEFIR_BIN_DIR) $< $@

$(KEFIR_END2END_BIN_PATH)/%.test.asmgen.done: $(KEFIR_END2END_BIN_PATH)/%.asmgen.output
	@echo "Asmgen-Diff $^"
	@diff -u $(SOURCE_DIR)/tests/end2end/$*.asmgen.expected $<
	@touch $@

$(KEFIR_END2END_BIN_PATH)/%.test.also.asmgen.done: $(KEFIR_END2END_BIN_PATH)/%.also.asmgen.output
	@echo "Asmgen-Diff $^"
	@diff -u $(SOURCE_DIR)/tests/end2end/$*.also.asmgen.expected $<
	@touch $@

$(SOURCE_DIR)/tests/end2end/%.asmgen.expected: $(SOURCE_DIR)/tests/end2end/%.kefir.asmgen.c $(KEFIR_BIN_DIR)/kefir
	@echo "Rebuilding $@"
	@ASMGEN=yes \
	 AS="$(AS)" \
	 VALGRIND_TEST_OPTIONS="$(VALGRIND_TEST_OPTIONS)" \
	 USE_VALGRIND="$(USE_VALGRIND)" \
	 PLATFORM="$(PLATFORM)" \
	 $(SOURCE_DIR)/tests/end2end/compile.sh $(KEFIR_BIN_DIR) $< $@

$(SOURCE_DIR)/tests/end2end/%.also.asmgen.expected: $(SOURCE_DIR)/tests/end2end/%.kefir.c $(KEFIR_BIN_DIR)/kefir
	@echo "Rebuilding $@"
	@ASMGEN=yes \
	 AS="$(AS)" \
	 VALGRIND_TEST_OPTIONS="$(VALGRIND_TEST_OPTIONS)" \
	 USE_VALGRIND="$(USE_VALGRIND)" \
	 PLATFORM="$(PLATFORM)" \
	 $(SOURCE_DIR)/tests/end2end/compile.sh $(KEFIR_BIN_DIR) $< $@

KEFIR_END2END_ALL_HOST_SOURCES=$(wildcard $(SOURCE_DIR)/tests/end2end/*/*.host.c)
KEFIR_END2END_ALL_KEFIR_SOURCES=$(wildcard $(SOURCE_DIR)/tests/end2end/*/*.kefir.c)
KEFIR_END2END_ALL_ASMGEN_SOURCES=$(wildcard $(SOURCE_DIR)/tests/end2end/*/*.kefir.asmgen.c)
KEFIR_END2END_ALL_ASMGEN_MARKS=$(wildcard $(SOURCE_DIR)/tests/end2end/*/*.kefir.also.asmgen)

KEFIR_END2END_ALL_RUN_TEST_SOURCES := $(KEFIR_END2END_ALL_HOST_SOURCES)
KEFIR_END2END_ALL_RUN_TEST_SOURCES += $(KEFIR_END2END_ALL_KEFIR_SOURCES)
ifeq ($(KEFIR_TEST_USE_MUSL),yes)
KEFIR_END2END_ALL_RUN_TEST_SOURCES := $(filter-out $(foreach S,$(KEFIR_END2END_ALL_RUN_TEST_SOURCES),$(if $(findstring atomic,$S),$S)),$(KEFIR_END2END_ALL_RUN_TEST_SOURCES))
endif
KEFIR_END2END_ALL_RUN_TEST_DEPENDENCIES := $(KEFIR_END2END_ALL_RUN_TEST_SOURCES:$(SOURCE_DIR)/%.c=$(KEFIR_BIN_DIR)/%.d)
KEFIR_END2END_ALL_RUN_TEST_COMPILE_DEPS := $(KEFIR_END2END_ALL_RUN_TEST_SOURCES:$(SOURCE_DIR)/%.c=$(KEFIR_BIN_DIR)/%.deps)
KEFIR_END2END_ALL_RUN_TEST_OBJECTS := $(KEFIR_END2END_ALL_RUN_TEST_SOURCES:$(SOURCE_DIR)/%.c=$(KEFIR_BIN_DIR)/%.o)
KEFIR_END2END_ALL_RUN_TEST_DIRS := $(dir $(KEFIR_END2END_ALL_RUN_TEST_SOURCES))
KEFIR_END2END_ALL_RUN_TEST_BINARIES := $(KEFIR_END2END_ALL_RUN_TEST_DIRS:$(SOURCE_DIR)/%/=$(KEFIR_BIN_DIR)/%.test)
KEFIR_END2END_ALL_RUN_TESTS := $(KEFIR_END2END_ALL_RUN_TEST_DIRS:$(SOURCE_DIR)/%/=$(KEFIR_BIN_DIR)/%.test.done)

KEFIR_END2END_ASMGEN_TEST_EXPECTED := $(KEFIR_END2END_ALL_ASMGEN_SOURCES:$(SOURCE_DIR)/%.kefir.asmgen.c=$(SOURCE_DIR)/%.asmgen.expected)
KEFIR_END2END_ASMGEN_TEST_RESULTS := $(KEFIR_END2END_ALL_ASMGEN_SOURCES:$(SOURCE_DIR)/%.kefir.asmgen.c=$(KEFIR_BIN_DIR)/%.asmgen.output)
KEFIR_END2END_ASMGEN_TESTS := $(KEFIR_END2END_ALL_ASMGEN_SOURCES:$(SOURCE_DIR)/%.kefir.asmgen.c=$(KEFIR_BIN_DIR)/%.test.asmgen.done)

KEFIR_END2END_ASMGEN_TEST_EXPECTED += $(KEFIR_END2END_ALL_ASMGEN_MARKS:$(SOURCE_DIR)/%.kefir.also.asmgen=$(SOURCE_DIR)/%.also.asmgen.expected)
KEFIR_END2END_ASMGEN_TEST_RESULTS += $(KEFIR_END2END_ALL_ASMGEN_MARKS:$(SOURCE_DIR)/%.kefir.also.asmgen=$(KEFIR_BIN_DIR)/%.also.asmgen.output)
KEFIR_END2END_ASMGEN_TESTS += $(KEFIR_END2END_ALL_ASMGEN_MARKS:$(SOURCE_DIR)/%.kefir.also.asmgen=$(KEFIR_BIN_DIR)/%.test.also.asmgen.done)

COMPILE_DEPS += $(KEFIR_END2END_ALL_RUN_TEST_COMPILE_DEPS)
DEPENDENCIES += $(KEFIR_END2END_ALL_RUN_TEST_DEPENDENCIES)
TEST_ARTIFACTS += $(KEFIR_END2END_ASMGEN_TEST_EXPECTED)
OBJECT_FILES += $(KEFIR_END2END_ALL_RUN_TEST_OBJECTS)
TEST_BINARIES += $(KEFIR_END2END_ALL_RUN_TEST_BINARIES)
TEST_RESULTS += $(KEFIR_END2END_ASMGEN_TEST_RESULTS)
TESTS += $(KEFIR_END2END_ALL_RUN_TESTS)
TESTS += $(KEFIR_END2END_ASMGEN_TESTS)
