
KEFIR_EXTERNAL_TEST_CGIT_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/cgit

KEFIR_EXTERNAL_TEST_CGIT_VERSION := 09d24d7cd0b7e85633f2f43808b12871bb209d69
KEFIR_EXTERNAL_TEST_CGIT_GIT_VERSION := 39bf06adf96da25b87c9aa7d35a32ef3683eb4a4
KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE_FILENAME := $(KEFIR_EXTERNAL_TEST_CGIT_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE_FILENAME := $(KEFIR_EXTERNAL_TEST_CGIT_GIT_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE := $(KEFIR_EXTERNAL_TEST_CGIT_DIR)/$(KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE := $(KEFIR_EXTERNAL_TEST_CGIT_DIR)/$(KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_CGIT_DIR)/cgit-$(KEFIR_EXTERNAL_TEST_CGIT_VERSION)
KEFIR_EXTERNAL_TEST_CGIT_GIT_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_CGIT_DIR)/git
KEFIR_EXTERNAL_TEST_CGIT_URL := https://github.com/zx2c4/cgit/archive/$(KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_CGIT_GIT_URL := https://github.com/git/git/archive/$(KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE_FILENAME)

KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE_SHA256 := b51315c85d2b2b9bbbd31cf436cb56923c108d1133b3dc4b1689ed2437a09a14
KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE_SHA256 := 7cb9a19f08b44935fc0b6afc6fef1ae172a2e6113d704d5e867a2e095c122386

$(KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_CGIT_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_CGIT_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_CGIT_GIT_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_CGIT_GIT_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE) $(KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_CGIT_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_CGIT_ARCHIVE_FILENAME)"
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)" && tar xvfz "$(realpath $(KEFIR_EXTERNAL_TEST_CGIT_GIT_ARCHIVE))"
	@rm -rf "$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)/git"
	@mv "$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)/git-$(KEFIR_EXTERNAL_TEST_CGIT_GIT_VERSION)" "$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)/git"
	@echo "Patching cgit $(KEFIR_EXTERNAL_TEST_CGIT_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)" && patch -p0 < "$(realpath $(SOURCE_DIR))/tests/external/cgit/cgit-$(KEFIR_EXTERNAL_TEST_CGIT_VERSION).patch"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)/cgit: $(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Building cgit $(KEFIR_EXTERNAL_TEST_CGIT_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE) CC="$(realpath $(KEFIR_EXE))" all

$(KEFIR_EXTERNAL_TEST_CGIT_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)/cgit
	@echo "Testing cgit $(KEFIR_EXTERNAL_TEST_CGIT_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_CGIT_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		bash -c 'set -o pipefail; $(MAKE) CC="$(realpath $(KEFIR_EXE))" test' | tee "$(shell realpath $@.tmp)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/cgit.test.done: $(KEFIR_EXTERNAL_TEST_CGIT_DIR)/tests.log
	@$(SOURCE_DIR)/tests/external/cgit/validate.sh "$(KEFIR_EXTERNAL_TEST_CGIT_DIR)/tests.log"
	@touch "$@"
	@echo "Successfully tested cgit $(KEFIR_EXTERNAL_TEST_CGIT_VERSION)"

EXTERNAL_TESTS_FAST_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/cgit.test.done
