
KEFIR_EXTERNAL_TEST_COREUTILS_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/coreutils

KEFIR_EXTERNAL_TEST_COREUTILS_VERSION := 9.5
KEFIR_EXTERNAL_TEST_GNULIB_VERSION := 1.0
KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE_FILENAME := v$(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_GNULIB_ARCHIVE_FILENAME := v$(KEFIR_EXTERNAL_TEST_GNULIB_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE := $(KEFIR_EXTERNAL_TEST_COREUTILS_DIR)/$(KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_GNULIB_ARCHIVE := $(KEFIR_EXTERNAL_TEST_COREUTILS_DIR)/$(KEFIR_EXTERNAL_TEST_GNULIB_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_COREUTILS_DIR)/coreutils-$(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION)
KEFIR_EXTERNAL_TEST_GNULIB_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_COREUTILS_DIR)/coreutils-$(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION)/gnulib-$(KEFIR_EXTERNAL_TEST_GNULIB_VERSION)
KEFIR_EXTERNAL_TEST_COREUTILS_URL := https://github.com/coreutils/coreutils/archive/refs/tags/v$(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_GNULIB_URL := https://github.com/coreutils/gnulib/archive/refs/tags/v$(KEFIR_EXTERNAL_TEST_GNULIB_VERSION).tar.gz

KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE_SHA256 := 7155d01f31bccf3387933ae3ba7ff1b313f15ed73ab13ccc5b211018a2d14636
KEFIR_EXTERNAL_TEST_GNUTLS_ARCHIVE_SHA256 := c3585637eeab2268400f26a7174862d72c416e9476a0ff4ec12ebce646a0f477

$(KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_COREUTILS_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_COREUTILS_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_GNULIB_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_GNULIB_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_GNULIB_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_GNUTLS_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_COREUTILS_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_COREUTILS_ARCHIVE_FILENAME)"
	@echo "Patching coreutils $(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)" && patch -p0 < "$(realpath $(SOURCE_DIR))/tests/external/coreutils/coreutils-9.5.patch"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_GNULIB_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)/.extracted $(KEFIR_EXTERNAL_TEST_GNULIB_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_GNULIB_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)" && tar xvfz "../$(KEFIR_EXTERNAL_TEST_GNULIB_ARCHIVE_FILENAME)"
	@echo "Patching gnulib $(KEFIR_EXTERNAL_TEST_GNULIB_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GNULIB_SOURCE_DIR)" && patch -p0 < "$(realpath $(SOURCE_DIR))/tests/external/coreutils/gnulib-1.0.patch"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)/configure: $(KEFIR_EXTERNAL_TEST_GNULIB_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Bootstrapping coreutils $(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		./bootstrap --no-git --gnulib-srcdir=$$PWD/gnulib-1.0

$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)/Makefile: $(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)/configure
	@echo "Configuring coreutils $(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		./configure

$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)/.done: $(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)/Makefile
	@echo "Building coreutils $(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE)
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_COREUTILS_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)/.done
	@echo "Testing coreutils $(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_COREUTILS_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE) check -j1 2>&1 | tee "$(shell realpath "$@.tmp")"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/coreutils.test.done: $(KEFIR_EXTERNAL_TEST_COREUTILS_DIR)/tests.log
	@$(SOURCE_DIR)/tests/external/coreutils/validate.sh "$(KEFIR_EXTERNAL_TEST_COREUTILS_DIR)/tests.log"
	@touch "$@"
	@echo "Coreutils $(KEFIR_EXTERNAL_TEST_COREUTILS_VERSION) test suite successfully finished"

EXTERNAL_TESTS_SLOW_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/coreutils.test.done
