KEFIR_EXTERNAL_TEST_CURL_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/curl

KEFIR_EXTERNAL_TEST_CURL_VERSION := 8.9.1
KEFIR_EXTERNAL_TEST_CURL_ARCHIVE_FILENAME := curl-$(KEFIR_EXTERNAL_TEST_CURL_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_CURL_ARCHIVE := $(KEFIR_EXTERNAL_TEST_CURL_DIR)/$(KEFIR_EXTERNAL_TEST_CURL_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_CURL_DIR)/curl-$(KEFIR_EXTERNAL_TEST_CURL_VERSION)
KEFIR_EXTERNAL_TEST_CURL_URL := https://github.com/curl/curl/releases/download/curl-$(subst .,_,$(KEFIR_EXTERNAL_TEST_CURL_VERSION))/curl-$(KEFIR_EXTERNAL_TEST_CURL_VERSION).tar.gz

KEFIR_EXTERNAL_TEST_CURL_ARCHIVE_SHA256 := 291124a007ee5111997825940b3876b3048f7d31e73e9caa681b80fe48b2dcd5

$(KEFIR_EXTERNAL_TEST_CURL_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_CURL_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_CURL_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_CURL_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_CURL_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_CURL_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_CURL_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_CURL_ARCHIVE_FILENAME)"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)/config.log: $(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Configuring curl $(KEFIR_EXTERNAL_TEST_CURL_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		CFLAGS="-O1 -fPIC -pie" \
		./configure --with-openssl

$(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)/src/curl: $(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)/config.log
	@echo "Building curl $(KEFIR_EXTERNAL_TEST_CURL_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		LC_ALL=C.UTF-8 \
		$(MAKE)

$(KEFIR_EXTERNAL_TEST_CURL_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)/src/curl
	@echo "Testing Git $(KEFIR_EXTERNAL_TEST_CURL_VERSION)..."
	echo "$(MAKEFLAGS)"
	@cd "$(KEFIR_EXTERNAL_TEST_CURL_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		LC_ALL=C.UTF-8 \
		$(MAKE) test TFLAGS='!165 !433 !537 !962 !963 !964 !965 !966 !967 !1448 !1560 !2046 !2047 $(shell echo $(MAKEFLAGS) | grep -Eo "\-j[0-9]+" || true)' 2>&1 | tee "$(shell realpath $@.tmp)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/curl.test.done: $(KEFIR_EXTERNAL_TEST_CURL_DIR)/tests.log
	@$(SOURCE_DIR)/tests/external/curl/validate.sh "$(KEFIR_EXTERNAL_TEST_CURL_DIR)/tests.log"
	@touch "$@"
	@echo "Successfully tested curl $(KEFIR_EXTERNAL_TEST_CURL_VERSION)"

EXTERNAL_TESTS_SLOW_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/curl.test.done
