
KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/gcc-bootstrap

KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION := 4.0.4
KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_FILENAME := gcc-$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION).tar.bz2
KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE := $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/gcc-$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)
KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR := $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/gcc-$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)-build
KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR := $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/gcc-$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)-install
KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_URL := https://ftp.mpi-inf.mpg.de/mirrors/gnu/mirror/gcc.gnu.org/pub/gcc/releases/gcc-$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)/$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_FILENAME)

KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_SHA256 := f3fde051c96d2fc084f6f4d6114ce4c1a079fcd1247d947d50a3d1641acafc47

$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)" && \
		bunzip2 -d < "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_FILENAME)" > "$(basename $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_FILENAME))"
	@cd "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)" && tar xvf "$(basename $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_ARCHIVE_FILENAME))"
	@echo "Applying $(SOURCE_DIR)/tests/external/gcc-bootstrap/gcc-4.0.4.patch"
	@patch "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_SOURCE_DIR)/gcc/config/i386/linux-unwind.h" < "$(SOURCE_DIR)/tests/external/gcc-bootstrap/gcc-4.0.4.patch"
	@rm -rf "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)" "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR)"
	@mkdir "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)"
	@mkdir "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR)"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)/Makefile: $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/.extracted $(KEFIR_EXE)
	@echo "Configuring gcc $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC=$(realpath $(KEFIR_EXE)) \
		KEFIR_AS=$(AS) \
		KEFIR_LD=$(LD) \
		$(realpath $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_SOURCE_DIR))/configure --enable-languages=c,c++ --disable-multilib --prefix=$(realpath $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR))

$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)/gcc/stage2/xgcc: $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)/Makefile
	@echo "Building gcc $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		KEFIR_AS=$(AS) \
		KEFIR_LD=$(LD) \
		$(MAKE) bootstrap

$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR)/bin/gcc: $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)/gcc/stage2/xgcc
	@echo "Installing gcc $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_BUILD_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		KEFIR_AS=$(AS) \
		KEFIR_LD=$(LD) \
		$(MAKE) install -j1

$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/test.log: $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR)/bin/gcc
	@echo "Validating gcc $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)..."
	@"$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR)/bin/gcc" \
		-O2 "$(SOURCE_DIR)/tests/external/gcc-bootstrap/test.c" -o "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/test"
	@cd "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)" && ./test "test case" | tee "test.log.tmp"
	@diff "$@.tmp" "$(SOURCE_DIR)/tests/external/gcc-bootstrap/test.log.expected"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/test.cxx.log: $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR)/bin/gcc
	@echo "Validating g++ $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)..."
	@"$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_INSTALL_DIR)/bin/g++" \
		-O2 "$(SOURCE_DIR)/tests/external/gcc-bootstrap/test.cpp" -o "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/test.cxx"
	@cd "$(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)" && ./test.cxx "test case" | tee "test.cxx.log.tmp"
	@diff "$@.tmp" "$(SOURCE_DIR)/tests/external/gcc-bootstrap/test.cxx.log.expected"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/gcc-bootstrap.test.done: $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/test.log $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_DIR)/test.cxx.log
	@touch "$@"
	@echo "Successfully validated gcc $(KEFIR_EXTERNAL_TEST_GCC_BOOTSTRAP_VERSION)"

EXTERNAL_TESTS_SLOW_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/gcc-bootstrap.test.done
