KEFIR_EXTERNAL_TEST_GIT_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/git

KEFIR_EXTERNAL_TEST_GIT_VERSION := 2.44.0
KEFIR_EXTERNAL_TEST_GIT_ARCHIVE_FILENAME := git-$(KEFIR_EXTERNAL_TEST_GIT_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_GIT_ARCHIVE := $(KEFIR_EXTERNAL_TEST_GIT_DIR)/$(KEFIR_EXTERNAL_TEST_GIT_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_GIT_DIR)/git-$(KEFIR_EXTERNAL_TEST_GIT_VERSION)
KEFIR_EXTERNAL_TEST_GIT_URL := https://www.kernel.org/pub/software/scm/git/git-$(KEFIR_EXTERNAL_TEST_GIT_VERSION).tar.gz

KEFIR_EXTERNAL_TEST_GIT_ARCHIVE_SHA256 := f9e36f085458fe9688fbbe7846b8c4770b13d161fcd8953655f36b2b85f06b76

$(KEFIR_EXTERNAL_TEST_GIT_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_GIT_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_GIT_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_GIT_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_GIT_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_GIT_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GIT_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_GIT_ARCHIVE_FILENAME)"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)/config.log: $(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Configuring git $(KEFIR_EXTERNAL_TEST_GIT_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		CFLAGS="-O1 -fPIC -pie" \
		./configure

$(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)/git: $(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)/config.log
	@echo "Building git $(KEFIR_EXTERNAL_TEST_GIT_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		$(MAKE) all

$(KEFIR_EXTERNAL_TEST_GIT_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)/git
	@echo "Testing Git $(KEFIR_EXTERNAL_TEST_GIT_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_GIT_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		$(MAKE) test 2>&1 | tee "$(shell realpath $@.tmp)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/git.test.done: $(KEFIR_EXTERNAL_TEST_GIT_DIR)/tests.log
	@$(SOURCE_DIR)/tests/external/git/validate.sh "$(KEFIR_EXTERNAL_TEST_GIT_DIR)/tests.log"
	@touch "$@"
	@echo "Successfully tested Git $(KEFIR_EXTERNAL_TEST_GIT_VERSION)"

EXTERNAL_TESTS_SLOW_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/git.test.done
