
KEFIR_EXTERNAL_TEST_LIBPNG_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/libpng

KEFIR_EXTERNAL_TEST_LIBPNG_VERSION := 1.6.45
KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE_FILENAME := v$(KEFIR_EXTERNAL_TEST_LIBPNG_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE := $(KEFIR_EXTERNAL_TEST_LIBPNG_DIR)/$(KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_LIBPNG_DIR)/libpng-$(KEFIR_EXTERNAL_TEST_LIBPNG_VERSION)
KEFIR_EXTERNAL_TEST_LIBPNG_URL := https://github.com/pnggroup/libpng/archive/refs/tags/v$(KEFIR_EXTERNAL_TEST_LIBPNG_VERSION).tar.gz

KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE_SHA256 := 7ff6898520645716ddc3d8381d97b6e02937b03da92e6fd0d7cf9d7d2b0da780

$(KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_LIBPNG_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_LIBPNG_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_LIBPNG_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_LIBPNG_ARCHIVE_FILENAME)"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)/Makefile: $(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Configuring libpng $(KEFIR_EXTERNAL_TEST_LIBPNG_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		./configure

$(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)/.libs/libpng16.so: $(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)/Makefile
	@echo "Building libpng $(KEFIR_EXTERNAL_TEST_LIBPNG_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE)

$(KEFIR_EXTERNAL_TEST_LIBPNG_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)/.libs/libpng16.so
	@echo "Testing libpng $(KEFIR_EXTERNAL_TEST_LIBPNG_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_LIBPNG_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE) check | tee "$(shell realpath $@.tmp)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/libpng.test.done: $(KEFIR_EXTERNAL_TEST_LIBPNG_DIR)/tests.log
	@$(SOURCE_DIR)/tests/external/libpng/validate.sh "$(KEFIR_EXTERNAL_TEST_LIBPNG_DIR)/tests.log"
	@touch "$@"
	@echo "Successfully tested libpng $(KEFIR_EXTERNAL_TEST_LIBPNG_VERSION)"

EXTERNAL_TESTS_FAST_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/libpng.test.done
