KEFIR_EXTERNAL_TEST_LUA_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/lua

KEFIR_EXTERNAL_TEST_LUA_VERSION := 5.4.6
KEFIR_EXTERNAL_TEST_LUA_ARCHIVE := lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_LUA_TESTS_ARCHIVE := lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION)-tests.tar.gz
KEFIR_EXTERNAL_TEST_LUA_URL := https://www.lua.org/ftp/lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_LUA_TESTS_URL := https://www.lua.org/tests/lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION)-tests.tar.gz

KEFIR_EXTERNAL_TEST_LUA_ARCHIVE_SHA256=7d5ea1b9cb6aa0b59ca3dde1c6adcb57ef83a1ba8e5432c0ecd06bf439b3ad88
KEFIR_EXTERNAL_TEST_LUA_TESTS_ARCHIVE_SHA256=63ed5f5bcfd15dfd2f72e04c2f8c10585bb8de9e558df62b335cb9e5c1a4ef34

KEFIR_EXTERAL_TEST_LUA_EXE := $(KEFIR_EXTERNAL_TEST_LUA_DIR)/lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION)/src/lua
KEFIR_EXTERAL_TEST_LUA_TEST_SUITE := $(KEFIR_EXTERNAL_TEST_LUA_DIR)/lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION)-tests/.extracted

KEFIR_EXTERNAL_TEST_LUA_CFLAGS := -O1 -fPIC -pie

$(KEFIR_EXTERNAL_TEST_LUA_DIR)/$(KEFIR_EXTERNAL_TEST_LUA_ARCHIVE):
	@mkdir -p $(shell dirname $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_LUA_URL)..."
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_LUA_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_LUA_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_LUA_DIR)/$(KEFIR_EXTERNAL_TEST_LUA_TESTS_ARCHIVE):
	@mkdir -p $(shell dirname $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_LUA_TESTS_URL)..."
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_LUA_TESTS_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_LUA_TESTS_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERAL_TEST_LUA_EXE): $(KEFIR_EXTERNAL_TEST_LUA_DIR)/$(KEFIR_EXTERNAL_TEST_LUA_ARCHIVE) $(KEFIR_EXE)
	@echo "Building Lua $(KEFIR_EXTERNAL_TEST_LUA_VERSION)..."
	@rm -rf "$(KEFIR_EXTERNAL_TEST_LUA_DIR)/lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION)"
	@cd "$(KEFIR_EXTERNAL_TEST_LUA_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_LUA_ARCHIVE)"
	@cd "$(KEFIR_EXTERNAL_TEST_LUA_DIR)/lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION)" && \
		LD_LIBRARY_PATH="$(shell $(REALPATH) $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(shell $(REALPATH) $(HEADERS_DIR)/runtime)" \
			$(MAKE) -f Makefile \
			CC="$(shell $(REALPATH) $(KEFIR_EXE))" \
			CFLAGS="$(KEFIR_EXTERNAL_TEST_LUA_CFLAGS)" \
			posix

$(KEFIR_EXTERAL_TEST_LUA_TEST_SUITE): $(KEFIR_EXTERNAL_TEST_LUA_DIR)/$(KEFIR_EXTERNAL_TEST_LUA_TESTS_ARCHIVE)
	@echo "Extracting Lua $(KEFIR_EXTERNAL_TEST_LUA_VERSION) tests..."
	@rm -rf "$(KEFIR_EXTERNAL_TEST_LUA_DIR)/lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION)-tests"
	@cd "$(KEFIR_EXTERNAL_TEST_LUA_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_LUA_TESTS_ARCHIVE)"
	@cd "$(KEFIR_EXTERNAL_TEST_LUA_DIR)/lua-$(KEFIR_EXTERNAL_TEST_LUA_VERSION)-tests" && patch < "$(shell $(REALPATH) $(SOURCE_DIR)/tests/external/lua/lua-5.4.6-tests.patch)"
	@touch "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/lua.test.done: $(KEFIR_EXTERAL_TEST_LUA_EXE) $(KEFIR_EXTERAL_TEST_LUA_TEST_SUITE)
	@echo "Running Lua $(KEFIR_EXTERNAL_TEST_LUA_VERSION) test suite..."
	@cd "$(shell dirname $(KEFIR_EXTERAL_TEST_LUA_TEST_SUITE))" && "$(shell $(REALPATH) $(KEFIR_EXTERAL_TEST_LUA_EXE))" -e"_U=true" all.lua
	@touch "$@"
	@echo "Lua $(KEFIR_EXTERNAL_TEST_LUA_VERSION) test suite successfully finished"

EXTERNAL_TESTS_BASE_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/lua.test.done
