
KEFIR_EXTERNAL_TEST_NASM_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/nasm

KEFIR_EXTERNAL_TEST_NASM_VERSION := 2.16.03
KEFIR_EXTERNAL_TEST_NASM_ARCHIVE_FILENAME := nasm-$(KEFIR_EXTERNAL_TEST_NASM_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_NASM_ARCHIVE := $(KEFIR_EXTERNAL_TEST_NASM_DIR)/$(KEFIR_EXTERNAL_TEST_NASM_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_NASM_DIR)/nasm-$(KEFIR_EXTERNAL_TEST_NASM_VERSION)
KEFIR_EXTERNAL_TEST_NASM_URL := https://www.nasm.us/pub/nasm/releasebuilds/$(KEFIR_EXTERNAL_TEST_NASM_VERSION)/nasm-$(KEFIR_EXTERNAL_TEST_NASM_VERSION).tar.gz

KEFIR_EXTERNAL_TEST_NASM_ARCHIVE_SHA256 := 5bc940dd8a4245686976a8f7e96ba9340a0915f2d5b88356874890e207bdb581

$(KEFIR_EXTERNAL_TEST_NASM_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_NASM_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_NASM_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_NASM_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_NASM_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_NASM_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_NASM_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_NASM_ARCHIVE_FILENAME)"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)/Makefile: $(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Configuring nasm $(KEFIR_EXTERNAL_TEST_NASM_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		./configure

$(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)/nasm: $(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)/Makefile
	@echo "Building nasm $(KEFIR_EXTERNAL_TEST_NASM_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE) everything

$(KEFIR_EXTERNAL_TEST_NASM_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)/nasm
	@echo "Testing nasm $(KEFIR_EXTERNAL_TEST_NASM_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_NASM_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE) -C test golden test diff -j1 | tee "$(shell realpath $@.tmp)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/nasm.test.done: $(KEFIR_EXTERNAL_TEST_NASM_DIR)/tests.log
	@$(SOURCE_DIR)/tests/external/nasm/validate.sh "$(KEFIR_EXTERNAL_TEST_NASM_DIR)/tests.log"
	@touch "$@"
	@echo "Successfully tested nasm $(KEFIR_EXTERNAL_TEST_NASM_VERSION)"

EXTERNAL_TESTS_FAST_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/nasm.test.done
