
KEFIR_EXTERNAL_TEST_OPENSSL_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/openssl

KEFIR_EXTERNAL_TEST_OPENSSL_VERSION := 3.4.0
KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE_FILENAME := openssl-$(KEFIR_EXTERNAL_TEST_OPENSSL_VERSION).tar.gz
KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE := $(KEFIR_EXTERNAL_TEST_OPENSSL_DIR)/$(KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_OPENSSL_DIR)/openssl-$(KEFIR_EXTERNAL_TEST_OPENSSL_VERSION)
KEFIR_EXTERNAL_TEST_OPENSSL_URL := https://github.com/openssl/openssl/releases/download/openssl-$(KEFIR_EXTERNAL_TEST_OPENSSL_VERSION)/openssl-$(KEFIR_EXTERNAL_TEST_OPENSSL_VERSION).tar.gz

KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE_SHA256 := e15dda82fe2fe8139dc2ac21a36d4ca01d5313c75f99f46c4e8a27709b7294bf

$(KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_OPENSSL_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_OPENSSL_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_OPENSSL_DIR)" && tar xvfz "$(KEFIR_EXTERNAL_TEST_OPENSSL_ARCHIVE_FILENAME)"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)/Makefile: $(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Configuring openssl $(KEFIR_EXTERNAL_TEST_OPENSSL_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		./Configure

$(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)/libssl.so.3: $(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)/Makefile
	@echo "Building openssl $(KEFIR_EXTERNAL_TEST_OPENSSL_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE)

$(KEFIR_EXTERNAL_TEST_OPENSSL_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)/libssl.so.3
	@echo "Testing openssl $(KEFIR_EXTERNAL_TEST_OPENSSL_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_OPENSSL_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		$(MAKE) test 2>&1 | tee "$(shell realpath "$@.tmp")"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/openssl.test.done: $(KEFIR_EXTERNAL_TEST_OPENSSL_DIR)/tests.log
	@$(SOURCE_DIR)/tests/external/openssl/validate.sh "$(KEFIR_EXTERNAL_TEST_OPENSSL_DIR)/tests.log"
	@touch "$@"
	@echo "OpenSSL $(KEFIR_EXTERNAL_TEST_OPENSSL_VERSION) test suite successfully finished"

EXTERNAL_TESTS_SLOW_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/openssl.test.done
