
KEFIR_EXTERNAL_TEST_PYTHON_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/python

KEFIR_EXTERNAL_TEST_PYTHON_VERSION := 3.13.1
KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE_FILENAME := python-$(KEFIR_EXTERNAL_TEST_PYTHON_VERSION).tar.xz
KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE := $(KEFIR_EXTERNAL_TEST_PYTHON_DIR)/$(KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_PYTHON_DIR)/Python-$(KEFIR_EXTERNAL_TEST_PYTHON_VERSION)
KEFIR_EXTERNAL_TEST_PYTHON_URL := https://www.python.org/ftp/python/$(KEFIR_EXTERNAL_TEST_PYTHON_VERSION)/Python-$(KEFIR_EXTERNAL_TEST_PYTHON_VERSION).tar.xz

KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE_SHA256 := 9cf9427bee9e2242e3877dd0f6b641c1853ca461f39d6503ce260a59c80bf0d9

$(KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_PYTHON_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_PYTHON_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_PYTHON_DIR)" && xz -d "$(KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE_FILENAME)" --stdout > "$(basename $(KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE_FILENAME))"
	@cd "$(KEFIR_EXTERNAL_TEST_PYTHON_DIR)" && tar xvf "$(basename $(KEFIR_EXTERNAL_TEST_PYTHON_ARCHIVE_FILENAME))"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)/Makefile: $(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Configuring python $(KEFIR_EXTERNAL_TEST_PYTHON_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		LC_ALL=C.UTF-8 \
		./configure

$(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)/python: $(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)/Makefile
	@echo "Building python $(KEFIR_EXTERNAL_TEST_PYTHON_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		LC_ALL=C.UTF-8 \
		$(MAKE)

$(KEFIR_EXTERNAL_TEST_PYTHON_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)/python
	@echo "Testing python $(KEFIR_EXTERNAL_TEST_PYTHON_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_PYTHON_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		LC_ALL=C.UTF-8 \
		bash -c "ulimit -s unlimited && ./python -m test -x test_gdb test_epoll test_external_inspection test_perf_profiler test_selectors test_ssl test_poplib test_socketserver" | tee "../tests.log.tmp"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TESTS_DIR)/python.test.done: $(KEFIR_EXTERNAL_TEST_PYTHON_DIR)/tests.log
	@$(SOURCE_DIR)/tests/external/python/validate.sh "$(KEFIR_EXTERNAL_TEST_PYTHON_DIR)/tests.log"
	@touch "$@"
	@echo "Python $(KEFIR_EXTERNAL_TEST_PYTHON_VERSION) test suite successfully finished"

EXTERNAL_TESTS_SLOW_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/python.test.done
