
KEFIR_EXTERNAL_TEST_SQLITE_DIR := $(KEFIR_EXTERNAL_TESTS_DIR)/sqlite

KEFIR_EXTERNAL_TEST_SQLITE_VERSION := 3470200
KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE_FILENAME := sqlite-src-$(KEFIR_EXTERNAL_TEST_SQLITE_VERSION).zip
KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE := $(KEFIR_EXTERNAL_TEST_SQLITE_DIR)/$(KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE_FILENAME)
KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR := $(KEFIR_EXTERNAL_TEST_SQLITE_DIR)/sqlite-src-$(KEFIR_EXTERNAL_TEST_SQLITE_VERSION)
KEFIR_EXTERNAL_TEST_SQLITE_URL := https://www.sqlite.org/2024/sqlite-src-$(KEFIR_EXTERNAL_TEST_SQLITE_VERSION).zip

KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE_SHA256 := e6a471f1238225f34c2c48c5601b54024cc538044368230f59ff0672be1fc623

$(KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE):
	@mkdir -p $(dir $@)
	@echo "Downloading $(KEFIR_EXTERNAL_TEST_SQLITE_URL)"
	@wget -O "$@.tmp" "$(KEFIR_EXTERNAL_TEST_SQLITE_URL)"
	@$(SCRIPTS_DIR)/checksum_sha256.sh "$@.tmp" "$(KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE_SHA256)"
	@mv "$@.tmp" "$@"

$(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)/.extracted: $(KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE)
	@echo "Extracting $(KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE_FILENAME)..."
	@cd "$(KEFIR_EXTERNAL_TEST_SQLITE_DIR)" && unzip -o "$(KEFIR_EXTERNAL_TEST_SQLITE_ARCHIVE_FILENAME)"
	@touch "$@"

$(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)/Makefile: $(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)/.extracted $(KEFIR_EXE)
	@echo "Configuring sqlite $(KEFIR_EXTERNAL_TEST_SQLITE_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		CC="$(realpath $(KEFIR_EXE))" \
		./configure

$(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)/sqlite3: $(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)/Makefile
	@echo "Building sqlite $(KEFIR_EXTERNAL_TEST_SQLITE_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		$(MAKE)

$(KEFIR_EXTERNAL_TEST_SQLITE_DIR)/tests.log: $(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)/sqlite3
	@echo "Testing sqlite $(KEFIR_EXTERNAL_TEST_SQLITE_VERSION)..."
	@cd "$(KEFIR_EXTERNAL_TEST_SQLITE_SOURCE_DIR)" && \
		LD_LIBRARY_PATH="$(realpath $(LIB_DIR)):$$LD_LIBRARY_PATH" \
		KEFIR_RTINC="$(realpath $(HEADERS_DIR))/kefir/runtime" \
		$(MAKE) tcltest | tee "../$(notdir $@)"

$(KEFIR_EXTERNAL_TESTS_DIR)/sqlite.test.done: $(KEFIR_EXTERNAL_TEST_SQLITE_DIR)/tests.log
	@"$(SOURCE_DIR)/tests/external/sqlite/validate.sh" $(KEFIR_EXTERNAL_TEST_SQLITE_DIR)/tests.log
	@touch "$@"
	@echo "Successfully tested SQLite3 $(KEFIR_EXTERNAL_TEST_SQLITE_VERSION)"

EXTERNAL_TESTS_FAST_SUITE += $(KEFIR_EXTERNAL_TESTS_DIR)/sqlite.test.done
