/*
    SPDX-License-Identifier: GPL-3.0

    Copyright (C) 2020-2025  Jevgenijs Protopopovs

    This file is part of Kefir project.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "kefir/ir/function.h"
#include "kefir/ir/module.h"
#include "kefir/ir/builder.h"
#include "kefir/core/mem.h"
#include "kefir/core/util.h"
#include "kefir/test/codegen.h"
#include "kefir/test/module_shim.h"

kefir_result_t kefir_int_test(struct kefir_mem *mem) {
    struct kefir_test_codegen codegen;
    kefir_test_codegen_init(mem, &codegen, stdout, NULL);

    struct kefir_ir_module module;
    REQUIRE_OK(kefir_ir_module_alloc(mem, &module));

    kefir_id_t int8type_id;
    struct kefir_ir_type *int8type = kefir_ir_module_new_type(mem, &module, 1, &int8type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, int8type, KEFIR_IR_TYPE_INT8, 0, 0));
    struct kefir_ir_data *int8data1 =
        kefir_ir_module_new_named_data(mem, &module, "int8_1", KEFIR_IR_DATA_GLOBAL_STORAGE, int8type_id);
    REQUIRE_OK(kefir_ir_data_set_integer(mem, int8data1, 0, 104));
    REQUIRE_OK(kefir_ir_data_finalize(mem, int8data1));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "int8_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));
    REQUIRE_OK(kefir_ir_data_finalize(
        mem, kefir_ir_module_new_named_data(mem, &module, "int8_2", KEFIR_IR_DATA_GLOBAL_STORAGE, int8type_id)));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "int8_2", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t int16type_id;
    struct kefir_ir_type *int16type = kefir_ir_module_new_type(mem, &module, 1, &int16type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, int16type, KEFIR_IR_TYPE_INT16, 0, 0));
    struct kefir_ir_data *int16data1 =
        kefir_ir_module_new_named_data(mem, &module, "int16_1", KEFIR_IR_DATA_GLOBAL_STORAGE, int16type_id);
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "int16_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, int16data1, 0, 2589));
    REQUIRE_OK(kefir_ir_data_finalize(mem, int16data1));
    REQUIRE_OK(kefir_ir_data_finalize(
        mem, kefir_ir_module_new_named_data(mem, &module, "int16_2", KEFIR_IR_DATA_GLOBAL_STORAGE, int16type_id)));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "int16_2", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t int32type_id;
    struct kefir_ir_type *int32type = kefir_ir_module_new_type(mem, &module, 1, &int32type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, int32type, KEFIR_IR_TYPE_INT32, 0, 0));
    struct kefir_ir_data *int32data1 =
        kefir_ir_module_new_named_data(mem, &module, "int32_1", KEFIR_IR_DATA_GLOBAL_STORAGE, int32type_id);
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "int32_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, int32data1, 0, 0x1ffff));
    REQUIRE_OK(kefir_ir_data_finalize(mem, int32data1));
    REQUIRE_OK(kefir_ir_data_finalize(
        mem, kefir_ir_module_new_named_data(mem, &module, "int32_2", KEFIR_IR_DATA_GLOBAL_STORAGE, int32type_id)));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "int32_2", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t int64type_id;
    struct kefir_ir_type *int64type = kefir_ir_module_new_type(mem, &module, 1, &int64type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, int64type, KEFIR_IR_TYPE_INT64, 0, 0));
    struct kefir_ir_data *int64data1 =
        kefir_ir_module_new_named_data(mem, &module, "int64_1", KEFIR_IR_DATA_GLOBAL_STORAGE, int64type_id);
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "int64_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, int64data1, 0, 0x1ffffffff));
    REQUIRE_OK(kefir_ir_data_finalize(mem, int64data1));
    REQUIRE_OK(kefir_ir_data_finalize(
        mem, kefir_ir_module_new_named_data(mem, &module, "int64_2", KEFIR_IR_DATA_GLOBAL_STORAGE, int64type_id)));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "int64_2", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t float32type_id;
    struct kefir_ir_type *float32type = kefir_ir_module_new_type(mem, &module, 1, &float32type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, float32type, KEFIR_IR_TYPE_FLOAT32, 0, 0));
    struct kefir_ir_data *float32data1 =
        kefir_ir_module_new_named_data(mem, &module, "float32_1", KEFIR_IR_DATA_GLOBAL_STORAGE, float32type_id);
    REQUIRE_OK(kefir_ir_data_set_float32(mem, float32data1, 0, 3.14159));
    REQUIRE_OK(kefir_ir_data_finalize(mem, float32data1));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "float32_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));
    REQUIRE_OK(kefir_ir_data_finalize(
        mem, kefir_ir_module_new_named_data(mem, &module, "float32_2", KEFIR_IR_DATA_GLOBAL_STORAGE, float32type_id)));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "float32_2", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t float64type_id;
    struct kefir_ir_type *float64type = kefir_ir_module_new_type(mem, &module, 1, &float64type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, float64type, KEFIR_IR_TYPE_FLOAT64, 0, 0));
    struct kefir_ir_data *float64data1 =
        kefir_ir_module_new_named_data(mem, &module, "float64_1", KEFIR_IR_DATA_GLOBAL_STORAGE, float64type_id);
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "float64_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));
    REQUIRE_OK(kefir_ir_data_set_float64(mem, float64data1, 0, 2.718281828));
    REQUIRE_OK(kefir_ir_data_finalize(mem, float64data1));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "float64_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));
    REQUIRE_OK(kefir_ir_data_finalize(
        mem, kefir_ir_module_new_named_data(mem, &module, "float64_2", KEFIR_IR_DATA_GLOBAL_STORAGE, float64type_id)));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "float64_2", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t struct1_type_id;
    struct kefir_ir_type *struct1_type = kefir_ir_module_new_type(mem, &module, 5, &struct1_type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, struct1_type, KEFIR_IR_TYPE_STRUCT, 0, 4));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, struct1_type, KEFIR_IR_TYPE_INT8, 0, 0));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, struct1_type, KEFIR_IR_TYPE_INT64, 0, 0));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, struct1_type, KEFIR_IR_TYPE_INT16, 0, 0));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, struct1_type, KEFIR_IR_TYPE_FLOAT32, 0, 0));
    struct kefir_ir_data *struct1_data1 =
        kefir_ir_module_new_named_data(mem, &module, "struct1_1", KEFIR_IR_DATA_GLOBAL_STORAGE, struct1_type_id);
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "struct1_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, struct1_data1, 1, 127));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, struct1_data1, 2, 0x2ffffffff));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, struct1_data1, 3, 4096));
    REQUIRE_OK(kefir_ir_data_set_float32(mem, struct1_data1, 4, 106.9994));
    REQUIRE_OK(kefir_ir_data_finalize(mem, struct1_data1));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "struct1_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t array1_type_id;
    struct kefir_ir_type *array1_type = kefir_ir_module_new_type(mem, &module, 6, &array1_type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, array1_type, KEFIR_IR_TYPE_ARRAY, 0, 3));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, array1_type, KEFIR_IR_TYPE_STRUCT, 0, 2));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, array1_type, KEFIR_IR_TYPE_ARRAY, 0, 2));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, array1_type, KEFIR_IR_TYPE_CHAR, 0, 0));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, array1_type, KEFIR_IR_TYPE_ARRAY, 0, 2));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, array1_type, KEFIR_IR_TYPE_FLOAT32, 0, 0));
    struct kefir_ir_data *array1_data =
        kefir_ir_module_new_named_data(mem, &module, "array1_1", KEFIR_IR_DATA_GLOBAL_STORAGE, array1_type_id);
    REQUIRE_OK(kefir_ir_data_set_integer(mem, array1_data, 3, 10));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, array1_data, 4, 20));
    REQUIRE_OK(kefir_ir_data_set_float32(mem, array1_data, 6, 3.4));
    REQUIRE_OK(kefir_ir_data_set_float32(mem, array1_data, 7, 4.5));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, array1_data, 17, 110));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, array1_data, 18, 120));
    REQUIRE_OK(kefir_ir_data_set_float32(mem, array1_data, 20, 40.56));
    REQUIRE_OK(kefir_ir_data_set_float32(mem, array1_data, 21, 56.74));
    REQUIRE_OK(kefir_ir_data_finalize(mem, array1_data));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "array1_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t union1_type_id;
    struct kefir_ir_type *union1_type = kefir_ir_module_new_type(mem, &module, 6, &union1_type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, union1_type, KEFIR_IR_TYPE_UNION, 0, 2));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, union1_type, KEFIR_IR_TYPE_STRUCT, 0, 2));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, union1_type, KEFIR_IR_TYPE_FLOAT32, 0, 0));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, union1_type, KEFIR_IR_TYPE_INT32, 0, 0));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, union1_type, KEFIR_IR_TYPE_ARRAY, 0, 8));
    REQUIRE_OK(kefir_irbuilder_type_append(mem, union1_type, KEFIR_IR_TYPE_CHAR, 0, 0));
    struct kefir_ir_data *union1_data =
        kefir_ir_module_new_named_data(mem, &module, "union1_1", KEFIR_IR_DATA_GLOBAL_STORAGE, union1_type_id);
    REQUIRE_OK(kefir_ir_data_set_float32(mem, union1_data, 2, 3.14));
    REQUIRE_OK(kefir_ir_data_set_integer(mem, union1_data, 3, 100500));
    REQUIRE_OK(kefir_ir_data_finalize(mem, union1_data));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "union1_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    kefir_id_t pointer1_type_id;
    struct kefir_ir_type *pointer1_type = kefir_ir_module_new_type(mem, &module, 1, &pointer1_type_id);
    REQUIRE_OK(kefir_irbuilder_type_append(mem, pointer1_type, KEFIR_IR_TYPE_WORD, 0, 0));
    struct kefir_ir_data *pointer1_data =
        kefir_ir_module_new_named_data(mem, &module, "pointer1_1", KEFIR_IR_DATA_GLOBAL_STORAGE, pointer1_type_id);
    REQUIRE_OK(kefir_ir_data_set_pointer(mem, pointer1_data, 0, "union1_1", 2));
    REQUIRE_OK(kefir_ir_data_finalize(mem, pointer1_data));
    REQUIRE_OK(kefir_ir_module_declare_global(mem, &module, "pointer1_1", KEFIR_IR_IDENTIFIER_GLOBAL_DATA));

    REQUIRE_OK(KEFIR_CODEGEN_TRANSLATE(mem, &codegen.iface, &module));
    REQUIRE_OK(KEFIR_CODEGEN_CLOSE(mem, &codegen.iface));
    REQUIRE_OK(kefir_ir_module_free(mem, &module));
    return KEFIR_OK;
}
