/*
    SPDX-License-Identifier: GPL-3.0

    Copyright (C) 2020-2025  Jevgenijs Protopopovs

    This file is part of Kefir project.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include "kefir/core/util.h"
#include "kefir/test/unit_test.h"
#include <assert.h>

#define TEST_CASES(_case, _separator)                                                        \
    _case(core_hashtree1) _separator _case(core_hashtree2)                                   \
    _separator _case(core_hashtree3)                                                         \
    _separator _case(core_bucketset1)                                                        \
    _separator _case(amd64_sysv_abi_data_test1)                                              \
    _separator _case(amd64_sysv_abi_data_test2)                                              \
    _separator _case(amd64_sysv_abi_data_test3)                                              \
    _separator _case(amd64_sysv_abi_data_test4)                                              \
    _separator _case(amd64_sysv_abi_data_test5)                                              \
    _separator _case(amd64_sysv_abi_data_test6)                                              \
    _separator _case(amd64_sysv_abi_data_test7)                                              \
    _separator _case(amd64_sysv_abi_data_test8)                                              \
    _separator _case(amd64_sysv_abi_data_test9)                                              \
    _separator _case(amd64_sysv_abi_classification_test1)                                    \
    _separator _case(amd64_sysv_abi_classification_test2)                                    \
    _separator _case(amd64_sysv_abi_classification_test3)                                    \
    _separator _case(amd64_sysv_abi_classification_test4)                                    \
    _separator _case(amd64_sysv_abi_classification_test5)                                    \
    _separator _case(amd64_sysv_abi_classification_test6)                                    \
    _separator _case(amd64_sysv_abi_classification_test7)                                    \
    _separator _case(amd64_sysv_abi_classification_test8)                                    \
    _separator _case(amd64_sysv_abi_classification_test9)                                    \
    _separator _case(amd64_sysv_abi_classification_test10)                                   \
    _separator _case(amd64_sysv_abi_classification_test11)                                   \
    _separator _case(amd64_sysv_abi_classification_test12)                                   \
    _separator _case(amd64_sysv_abi_classification_test13)                                   \
    _separator _case(amd64_sysv_abi_classification_test14)                                   \
    _separator _case(amd64_sysv_abi_classification_test15)                                   \
    _separator _case(amd64_sysv_abi_allocation_test1)                                        \
    _separator _case(amd64_sysv_abi_allocation_test2)                                        \
    _separator _case(amd64_sysv_abi_allocation_test3)                                        \
    _separator _case(amd64_sysv_abi_allocation_test4)                                        \
    _separator _case(amd64_sysv_abi_allocation_test5)                                        \
    _separator _case(amd64_sysv_abi_allocation_test6)                                        \
    _separator _case(ast_type_analysis_integer_promotion1)                                   \
    _separator _case(ast_type_analysis_bitfield_promotion1)                                  \
    _separator _case(ast_type_analysis_arithmetic_conversion1)                               \
    _separator _case(ast_type_analysis_arithmetic_conversion2)                               \
    _separator _case(ast_type_analysis_arithmetic_conversion3)                               \
    _separator _case(ast_ordinary_scope_objects1)                                            \
    _separator _case(ast_ordinary_scope_objects2)                                            \
    _separator _case(ast_ordinary_scope_objects3)                                            \
    _separator _case(ast_ordinary_scope_objects4)                                            \
    _separator _case(ast_ordinary_scope_objects5)                                            \
    _separator _case(ast_ordinary_scope_objects6)                                            \
    _separator _case(ast_ordinary_scope_objects7)                                            \
    _separator _case(ast_ordinary_scope_objects8)                                            \
    _separator _case(ast_ordinary_scope_objects9)                                            \
    _separator _case(ast_ordinary_scope_objects10)                                           \
    _separator _case(ast_ordinary_scope_objects11)                                           \
    _separator _case(ast_ordinary_scope_objects12)                                           \
    _separator _case(ast_ordinary_scope_objects13)                                           \
    _separator _case(ast_ordinary_scope_objects14)                                           \
    _separator _case(ast_ordinary_scope_objects15)                                           \
    _separator _case(ast_type_construction1)                                                 \
    _separator _case(ast_type_construction2)                                                 \
    _separator _case(ast_type_construction3)                                                 \
    _separator _case(ast_type_construction4)                                                 \
    _separator _case(ast_type_construction5)                                                 \
    _separator _case(ast_type_construction6)                                                 \
    _separator _case(ast_type_construction7)                                                 \
    _separator _case(ast_type_construction8)                                                 \
    _separator _case(ast_nodes_constants)                                                    \
    _separator _case(ast_nodes_identifiers)                                                  \
    _separator _case(ast_nodes_string_literals_multibyte)                                    \
    _separator _case(ast_nodes_string_literals_unicode8)                                     \
    _separator _case(ast_nodes_string_literals_unicode16)                                    \
    _separator _case(ast_nodes_string_literals_unicode32)                                    \
    _separator _case(ast_nodes_string_literals_wide)                                         \
    _separator _case(ast_nodes_type_name)                                                    \
    _separator _case(ast_nodes_array_subscripts)                                             \
    _separator _case(ast_nodes_function_calls)                                               \
    _separator _case(ast_nodes_builtins)                                                     \
    _separator _case(ast_nodes_struct_members)                                               \
    _separator _case(ast_nodes_unary_operations)                                             \
    _separator _case(ast_nodes_binary_operations)                                            \
    _separator _case(ast_nodes_generic_selections)                                           \
    _separator _case(ast_nodes_cast_operators)                                               \
    _separator _case(ast_nodes_conditional_operators)                                        \
    _separator _case(ast_nodes_assignment_operators)                                         \
    _separator _case(ast_nodes_comma_operators)                                              \
    _separator _case(ast_nodes_compound_literals)                                            \
    _separator _case(ast_nodes_init_declarators1)                                            \
    _separator _case(ast_nodes_static_assertions1)                                           \
    _separator _case(ast_nodes_labeled_statements1)                                          \
    _separator _case(ast_nodes_case_statements1)                                             \
    _separator _case(ast_nodes_expression_statements1)                                       \
    _separator _case(ast_nodes_extension)                                                    \
    _separator _case(ast_nodes_label_address)                                                \
    _separator _case(ast_nodes_goto_address_statements1)                                     \
    _separator _case(ast_nodes_statement_expressions1)                                       \
    _separator _case(ast_nodes_attribute_list1)                                              \
    _separator _case(ast_nodes_inline_assembly1)                                             \
    _separator _case(ast_type_compatibility1)                                                \
    _separator _case(ast_type_basic_compatibility)                                           \
    _separator _case(ast_type_enum_compatibility)                                            \
    _separator _case(ast_type_pointer_compatibility)                                         \
    _separator _case(ast_type_qualified_compatibility)                                       \
    _separator _case(ast_type_struct_compatibility)                                          \
    _separator _case(ast_type_union_compatibility)                                           \
    _separator _case(ast_type_array_compatibility)                                           \
    _separator _case(ast_type_function_compatibility)                                        \
    _separator _case(ast_type_basic_composite)                                               \
    _separator _case(ast_type_pointer_composite)                                             \
    _separator _case(ast_type_enum_composite)                                                \
    _separator _case(ast_type_qualified_composite)                                           \
    _separator _case(ast_type_struct_composite)                                              \
    _separator _case(ast_type_union_composite)                                               \
    _separator _case(ast_type_array_composite)                                               \
    _separator _case(ast_type_function_composite)                                            \
    _separator _case(ast_type_scope1)                                                        \
    _separator _case(ast_type_scope2)                                                        \
    _separator _case(ast_type_scope3)                                                        \
    _separator _case(ast_type_variably_modified1)                                            \
    _separator _case(ast_type_variably_modified2)                                            \
    _separator _case(ast_type_variably_modified_analysis1)                                   \
    _separator _case(ast_declaration_variably_modified_structure_field)                      \
    _separator _case(ast_ordinary_constant_scope1)                                           \
    _separator _case(ast_ordinary_typedef_scope1)                                            \
    _separator _case(ast_ordinary_scope_composite_objects_external)                          \
    _separator _case(ast_ordinary_scope_composite_objects_external_declaration)              \
    _separator _case(ast_ordinary_scope_composite_objects_thread_local_external)             \
    _separator _case(ast_ordinary_scope_composite_objects_thread_local_external_declaration) \
    _separator _case(ast_ordinary_scope_composite_objects_static)                            \
    _separator _case(ast_ordinary_scope_composite_objects_static_thread_local)               \
    _separator _case(ast_ordinary_scope_composite_objects_local_external)                    \
    _separator _case(ast_ordinary_scope_composite_objects_local_thread_local_external)       \
    _separator _case(ast_ordinary_function_scope1)                                           \
    _separator _case(ast_ordinary_function_scope2)                                           \
    _separator _case(ast_ordinary_function_scope3)                                           \
    _separator _case(ast_ordinary_function_scope4)                                           \
    _separator _case(ast_ordinary_function_scope5)                                           \
    _separator _case(ast_ordinary_inline_functions1)                                         \
    _separator _case(ast_ordinary_inline_functions2)                                         \
    _separator _case(ast_ordinary_inline_functions3)                                         \
    _separator _case(ast_ordinary_scope_objects_init1)                                       \
    _separator _case(ast_ordinary_scope_objects_init2)                                       \
    _separator _case(ast_ordinary_scope_objects_init3)                                       \
    _separator _case(ast_ordinary_scope_objects_init4)                                       \
    _separator _case(ast_ordinary_scope_objects_init5)                                       \
    _separator _case(ast_ordinary_scope_objects_init6)                                       \
    _separator _case(ast_ordinary_scope_objects_init7)                                       \
    _separator _case(ast_ordinary_scope_objects_init8)                                       \
    _separator _case(ast_ordinary_scope_objects_local_init1)                                 \
    _separator _case(ast_ordinary_scope_objects_local_init2)                                 \
    _separator _case(ast_ordinary_scope_objects_local_init3)                                 \
    _separator _case(ast_ordinary_scope_objects_local_init4)                                 \
    _separator _case(ast_node_analysis_constants)                                            \
    _separator _case(ast_node_analysis_string_literals_multibyte)                            \
    _separator _case(ast_node_analysis_string_literals_unicode8)                             \
    _separator _case(ast_node_analysis_string_literals_unicode16)                            \
    _separator _case(ast_node_analysis_string_literals_unicode32)                            \
    _separator _case(ast_node_analysis_string_literals_wide)                                 \
    _separator _case(ast_node_analysis_identifiers)                                          \
    _separator _case(ast_node_analysis_array_subscripts)                                     \
    _separator _case(ast_node_analysis_struct_members1)                                      \
    _separator _case(ast_node_analysis_struct_members2)                                      \
    _separator _case(ast_node_analysis_indirect_struct_members1)                             \
    _separator _case(ast_node_analysis_indirect_struct_members2)                             \
    _separator _case(ast_node_analysis_function_calls)                                       \
    _separator _case(ast_node_analysis_function_calls1)                                      \
    _separator _case(ast_node_analysis_function_calls2)                                      \
    _separator _case(ast_node_analysis_function_calls3)                                      \
    _separator _case(ast_node_analysis_function_calls4)                                      \
    _separator _case(ast_node_analysis_function_call_qualified_rvalues)                      \
    _separator _case(ast_node_analysis_builtins1)                                            \
    _separator _case(ast_node_analysis_builtins2)                                            \
    _separator _case(ast_node_analysis_builtins3)                                            \
    _separator _case(ast_node_analysis_builtins4)                                            \
    _separator _case(ast_node_analysis_unary_operation_arithmetic)                           \
    _separator _case(ast_node_analysis_unary_operation_address)                              \
    _separator _case(ast_node_analysis_unary_operation_indirect)                             \
    _separator _case(ast_node_analysis_unary_operation_incdec)                               \
    _separator _case(ast_node_analysis_unary_operation_sizeof)                               \
    _separator _case(ast_node_analysis_unary_operation_alignof)                              \
    _separator _case(ast_node_analysis_type_name)                                            \
    _separator _case(ast_node_analysis_generic_selections)                                   \
    _separator _case(ast_node_analysis_cast_operators)                                       \
    _separator _case(ast_node_analysis_cast_operator_qualified_rvalue)                       \
    _separator _case(ast_node_analysis_multiplicative_operators)                             \
    _separator _case(ast_node_analysis_add_operator)                                         \
    _separator _case(ast_node_analysis_subtract_operator)                                    \
    _separator _case(ast_node_analysis_shift_operator)                                       \
    _separator _case(ast_node_analysis_relational_operators)                                 \
    _separator _case(ast_node_analysis_equality_operators)                                   \
    _separator _case(ast_node_analysis_bitwise_operators)                                    \
    _separator _case(ast_node_analysis_logical_operators)                                    \
    _separator _case(ast_node_analysis_conditional_operator1)                                \
    _separator _case(ast_node_analysis_conditional_operator2)                                \
    _separator _case(ast_node_analysis_conditional_operator3)                                \
    _separator _case(ast_node_analysis_conditional_operator4)                                \
    _separator _case(ast_node_analysis_conditional_operator5)                                \
    _separator _case(ast_node_analysis_conditional_operator6)                                \
    _separator _case(ast_node_analysis_simple_assignment_operator1)                          \
    _separator _case(ast_node_analysis_simple_assignment_operator2)                          \
    _separator _case(ast_node_analysis_simple_assignment_operator3)                          \
    _separator _case(ast_node_analysis_simple_assignment_operator4)                          \
    _separator _case(ast_node_analysis_simple_assignment_operator5)                          \
    _separator _case(ast_node_analysis_simple_assignment_operator6)                          \
    _separator _case(ast_node_analysis_simple_assignment_operator7)                          \
    _separator _case(ast_node_analysis_compound_assignment_operator1)                        \
    _separator _case(ast_node_analysis_compound_assignment_operator2)                        \
    _separator _case(ast_node_analysis_compound_assignment_operator3)                        \
    _separator _case(ast_nodes_comma_operators)                                              \
    _separator _case(ast_node_analysis_compound_literal1)                                    \
    _separator _case(ast_node_analysis_compound_literal2)                                    \
    _separator _case(ast_node_analysis_compound_literal3)                                    \
    _separator _case(ast_node_analysis_compound_literal4)                                    \
    _separator _case(ast_node_analysis_inline_assembly1)                                     \
    _separator _case(ast_node_analysis_inline_assembly2)                                     \
    _separator _case(ast_type_traversal1)                                                    \
    _separator _case(ast_type_traversal2)                                                    \
    _separator _case(ast_type_traversal3)                                                    \
    _separator _case(ast_type_traversal4)                                                    \
    _separator _case(ast_type_traversal5)                                                    \
    _separator _case(ast_initializer_construction1)                                          \
    _separator _case(ast_initializer_construction2)                                          \
    _separator _case(ast_initializer_analysis1)                                              \
    _separator _case(ast_initializer_analysis2)                                              \
    _separator _case(ast_initializer_analysis3)                                              \
    _separator _case(ast_initializer_analysis4)                                              \
    _separator _case(ast_initializer_analysis5)                                              \
    _separator _case(ast_initializer_analysis_unicode8)                                      \
    _separator _case(ast_initializer_analysis_unicode16)                                     \
    _separator _case(ast_initializer_analysis_unicode32)                                     \
    _separator _case(ast_initializer_analysis_wide)                                          \
    _separator _case(ast_initializer_analysis_mixed_strings)                                 \
    _separator _case(ast_constant_expression_constant1)                                      \
    _separator _case(ast_constant_expression_identifier1)                                    \
    _separator _case(ast_constant_expression_string_literal1)                                \
    _separator _case(ast_constant_expression_generic_selection1)                             \
    _separator _case(ast_constant_expression_unary_operations1)                              \
    _separator _case(ast_constant_expression_unary_operations2)                              \
    _separator _case(ast_constant_expression_unary_operations3)                              \
    _separator _case(ast_constant_expression_unary_operations4)                              \
    _separator _case(ast_constant_expression_unary_operations5)                              \
    _separator _case(ast_constant_expression_unary_operations6)                              \
    _separator _case(ast_constant_expression_unary_operations7)                              \
    _separator _case(ast_constant_expression_binary_operations1)                             \
    _separator _case(ast_constant_expression_binary_operations2)                             \
    _separator _case(ast_constant_expression_binary_operations3)                             \
    _separator _case(ast_constant_expression_binary_operations4)                             \
    _separator _case(ast_constant_expression_conditional_operator1)                          \
    _separator _case(ast_constant_expression_conditional_operator2)                          \
    _separator _case(ast_constant_expression_conditional_operator3)                          \
    _separator _case(ast_constant_expression_cast_operator1)                                 \
    _separator _case(ast_constant_expression_cast_operator2)                                 \
    _separator _case(ast_translator_environment1)                                            \
    _separator _case(ast_translator_environment2)                                            \
    _separator _case(ast_alignment_default)                                                  \
    _separator _case(ast_alignment_as_type)                                                  \
    _separator _case(ast_alignment_as_const_expr)                                            \
    _separator _case(ast_type_analysis_scalars)                                              \
    _separator _case(ast_type_analysis_pointers_qualifications)                              \
    _separator _case(ast_type_analysis_enumerations)                                         \
    _separator _case(ast_type_analysis_arrays)                                               \
    _separator _case(ast_type_analysis_structs)                                              \
    _separator _case(ast_type_analysis_functions)                                            \
    _separator _case(ast_type_analysis_flexible_arrays)                                      \
    _separator _case(ast_bitfield_translator1)                                               \
    _separator _case(ast_bitfield_translator2)                                               \
    _separator _case(ast_bitfield_translator3)                                               \
    _separator _case(ast_bitfield_translator4)                                               \
    _separator _case(ast_bitfield_translator5)                                               \
    _separator _case(ast_declarator_specifier_construction1)                                 \
    _separator _case(ast_declarator_specifier_construction2)                                 \
    _separator _case(ast_declarator_specifier_construction3)                                 \
    _separator _case(ast_declarator_specifier_construction4)                                 \
    _separator _case(ast_declarator_specifier_construction5)                                 \
    _separator _case(ast_declarator_specifier_construction6)                                 \
    _separator _case(ast_declarator_specifier_construction7)                                 \
    _separator _case(ast_declarator_specifier_construction8)                                 \
    _separator _case(ast_declarator_specifier_construction9)                                 \
    _separator _case(ast_declarator_specifier_construction10)                                \
    _separator _case(ast_declarator_specifier_construction11)                                \
    _separator _case(ast_declarator_construction1)                                           \
    _separator _case(ast_declarator_construction2)                                           \
    _separator _case(ast_declarator_construction3)                                           \
    _separator _case(ast_declarator_construction4)                                           \
    _separator _case(ast_declarator_construction5)                                           \
    _separator _case(ast_declarator_construction6)                                           \
    _separator _case(ast_declarator_construction7)                                           \
    _separator _case(ast_declarator_construction8)                                           \
    _separator _case(ast_declarator_construction9)                                           \
    _separator _case(ast_declarator_construction10)                                          \
    _separator _case(ast_declarator_construction11)                                          \
    _separator _case(ast_structure_declaration1)                                             \
    _separator _case(ast_structure_declaration2)                                             \
    _separator _case(ast_enum_declaration1)                                                  \
    _separator _case(ast_enum_declaration2)                                                  \
    _separator _case(ast_declarator_analysis1)                                               \
    _separator _case(ast_declarator_analysis2)                                               \
    _separator _case(ast_declarator_analysis3)                                               \
    _separator _case(ast_declarator_analysis4)                                               \
    _separator _case(ast_declarator_analysis5)                                               \
    _separator _case(ast_declarator_analysis6)                                               \
    _separator _case(ast_declarator_analysis7)                                               \
    _separator _case(ast_declarator_analysis8)                                               \
    _separator _case(ast_declarator_analysis9)                                               \
    _separator _case(ast_declarator_analysis10)                                              \
    _separator _case(ast_declarator_analysis11)                                              \
    _separator _case(ast_declarator_analysis12)                                              \
    _separator _case(ast_declarator_analysis13)                                              \
    _separator _case(ast_declarator_analysis14)                                              \
    _separator _case(ast_declarator_analysis15)                                              \
    _separator _case(ast_declarator_analysis16)                                              \
    _separator _case(ast_declarator_analysis17)                                              \
    _separator _case(ast_declarator_analysis18)                                              \
    _separator _case(ast_declarator_analysis19)                                              \
    _separator _case(ast_declarator_analysis20)                                              \
    _separator _case(ast_declarator_analysis21)                                              \
    _separator _case(ast_declarator_analysis22)                                              \
    _separator _case(ast_declarator_analysis23)                                              \
    _separator _case(ast_declarator_analysis24)                                              \
    _separator _case(ast_declarator_analysis25)                                              \
    _separator _case(ast_declaration_variably_modified1)                                     \
    _separator _case(ast_declaration_variably_modified2)                                     \
    _separator _case(ast_declaration_variably_modified3)                                     \
    _separator _case(ast_declaration_variably_modified4)                                     \
    _separator _case(ast_node_analysis_init_declarators1)                                    \
    _separator _case(ast_node_analysis_init_declarators2)                                    \
    _separator _case(ast_node_analysis_init_declarators3)                                    \
    _separator _case(ast_node_analysis_init_declarators4)                                    \
    _separator _case(ast_node_analysis_init_declarators5)                                    \
    _separator _case(ast_node_analysis_init_declarators6)                                    \
    _separator _case(ast_node_analysis_init_declarators7)                                    \
    _separator _case(ast_node_analysis_init_declarators8)                                    \
    _separator _case(ast_node_analysis_init_declarators9)                                    \
    _separator _case(ast_node_analysis_init_declarators10)                                   \
    _separator _case(ast_node_analysis_init_declarators11)                                   \
    _separator _case(ast_node_analysis_init_declarators12)                                   \
    _separator _case(ast_node_analysis_init_declarators13)                                   \
    _separator _case(ast_node_analysis_static_assertions1)                                   \
    _separator _case(ast_node_analysis_expression_statements1)                               \
    _separator _case(ast_node_analysis_labeled_statements1)                                  \
    _separator _case(ast_node_analysis_case_statements1)                                     \
    _separator _case(ast_nodes_compound_statements1)                                         \
    _separator _case(ast_nodes_conditional_statements1)                                      \
    _separator _case(ast_label_scope1)                                                       \
    _separator _case(ast_label_scope2)                                                       \
    _separator _case(ast_label_scope3)                                                       \
    _separator _case(ast_label_scope4)                                                       \
    _separator _case(ast_node_analysis_labeled_statements2)                                  \
    _separator _case(ast_node_analysis_compound_statements1)                                 \
    _separator _case(ast_node_analysis_compound_statements2)                                 \
    _separator _case(ast_node_analysis_compound_statements3)                                 \
    _separator _case(ast_node_analysis_compound_statements4)                                 \
    _separator _case(valueast_node_analysis_conditional_statements1)                         \
    _separator _case(ast_node_analysis_conditinal_statements2)                               \
    _separator _case(ast_node_analysis_conditinal_statements3)                               \
    _separator _case(ast_node_analysis_goto_address_statements1)                             \
    _separator _case(ast_node_analysis_statement_expressions1)                               \
    _separator _case(ast_flow_control_tree1)                                                 \
    _separator _case(ast_flow_control_tree2)                                                 \
    _separator _case(ast_flow_control_tree3)                                                 \
    _separator _case(ast_flow_control_tree4)                                                 \
    _separator _case(ast_flow_control_tree5)                                                 \
    _separator _case(ast_flow_control_tree6)                                                 \
    _separator _case(ast_flow_control_tree_vl_arrays1)                                       \
    _separator _case(ast_nodes_switch_statements1)                                           \
    _separator _case(ast_node_analysis_switch_statements1)                                   \
    _separator _case(ast_node_analysis_switch_statements2)                                   \
    _separator _case(ast_node_analysis_switch_statements3)                                   \
    _separator _case(ast_node_analysis_switch_statements4)                                   \
    _separator _case(ast_node_analysis_switch_statements5)                                   \
    _separator _case(ast_nodes_while_statements1)                                            \
    _separator _case(ast_nodes_while_statements2)                                            \
    _separator _case(ast_nodes_do_while_statements1)                                         \
    _separator _case(ast_nodes_do_while_statements2)                                         \
    _separator _case(ast_nodes_for_statements1)                                              \
    _separator _case(ast_nodes_for_statements2)                                              \
    _separator _case(ast_nodes_for_statements3)                                              \
    _separator _case(ast_nodes_for_statements4)                                              \
    _separator _case(ast_nodes_for_statements5)                                              \
    _separator _case(ast_node_analysis_while_statements1)                                    \
    _separator _case(ast_node_analysis_while_statements2)                                    \
    _separator _case(ast_node_analysis_do_while_statements1)                                 \
    _separator _case(ast_node_analysis_do_while_statements2)                                 \
    _separator _case(ast_node_analysis_for_statements1)                                      \
    _separator _case(ast_node_analysis_for_statements2)                                      \
    _separator _case(ast_node_analysis_for_statements3)                                      \
    _separator _case(ast_nodes_goto_statements1)                                             \
    _separator _case(ast_nodes_continue_statements)                                          \
    _separator _case(ast_nodes_break_statements)                                             \
    _separator _case(ast_nodes_return_statements1)                                           \
    _separator _case(ast_node_analysis_goto_statements1)                                     \
    _separator _case(ast_node_analysis_break_statements1)                                    \
    _separator _case(ast_node_analysis_break_statements2)                                    \
    _separator _case(ast_node_analysis_break_statements3)                                    \
    _separator _case(ast_node_analysis_break_statements4)                                    \
    _separator _case(ast_node_analysis_break_statements5)                                    \
    _separator _case(ast_node_analysis_break_statements6)                                    \
    _separator _case(ast_node_analysis_continue_statements1)                                 \
    _separator _case(ast_node_analysis_continue_statements2)                                 \
    _separator _case(ast_node_analysis_continue_statements3)                                 \
    _separator _case(ast_node_analysis_continue_statements4)                                 \
    _separator _case(ast_node_analysis_continue_statements5)                                 \
    _separator _case(ast_node_analysis_return_statements1)                                   \
    _separator _case(ast_translator_flow_control_point1)                                     \
    _separator _case(ast_nodes_function_definitions1)                                        \
    _separator _case(ast_nodes_translation_units1)                                           \
    _separator _case(ast_node_analysis_function_definitions1)                                \
    _separator _case(ast_node_analysis_function_definitions2)                                \
    _separator _case(ast_node_analysis_function_definitions3)                                \
    _separator _case(ast_node_analysis_translation_unit1)                                    \
    _separator _case(ast_node_analysis_translation_unit2)                                    \
    _separator _case(ast_nodes_declaration1)                                                 \
    _separator _case(ast_node_analysis_declaration1)                                         \
    _separator _case(ast_node_analysis_label_address)                                        \
    _separator _case(ast_context_preprocessor_extensions1)                                   \
    _separator _case(ast_context_global_extensions1)                                         \
    _separator _case(ast_context_local_extensions1)                                          \
    _separator _case(ast_context_function_decl_context_extensions1)                          \
    _separator _case(ast_analysis_extension_node1)                                           \
    _separator _case(ast_analysis_before_after_extensions)                                   \
    _separator _case(ast_translator_context_extensions1)                                     \
    _separator _case(ast_translator_context_local_extensions1)                               \
    _separator _case(parser_lexem_construction_sentinel)                                     \
    _separator _case(parser_lexem_construction_keyword)                                      \
    _separator _case(parser_lexem_construction_identifier)                                   \
    _separator _case(parser_lexem_construction_constants)                                    \
    _separator _case(parser_lexem_construction_string_literals)                              \
    _separator _case(parser_lexem_construction_unicode8_string_literals)                     \
    _separator _case(parser_lexem_construction_unicode16_string_literals)                    \
    _separator _case(parser_lexem_construction_unicode32_string_literals)                    \
    _separator _case(parser_lexem_construction_wide_string_literals)                         \
    _separator _case(parser_lexem_construction_punctuator)                                   \
    _separator _case(parser_lexem_construction_pp_whitespace)                                \
    _separator _case(parser_lexem_construction_pp_numbers)                                   \
    _separator _case(parser_lexem_construction_pp_header_name)                               \
    _separator _case(parser_lexem_move)                                                      \
    _separator _case(parser_lexem_copy)                                                      \
    _separator _case(parser_token_cursor1)                                                   \
    _separator _case(parser_block_scope1)                                                    \
    _separator _case(parser_scope1)                                                          \
    _separator _case(parser_extensions1)                                                     \
    _separator _case(lexer_source_cursor1)                                                   \
    _separator _case(lexer_source_location1)                                                 \
    _separator _case(lexer_source_cursor_newlines)                                           \
    _separator _case(preprocessor_macro_construction1)                                       \
    _separator _case(preprocessor_macro_user_scope1)                                         \
    _separator _case(cli_options1)                                                           \
    _separator _case(cli_options2)                                                           \
    _separator _case(cli_options3)                                                           \
    _separator _case(cli_options4)

TEST_CASES(DECLARE_CASE, ;);

#define RUN_CASE(_id) &_id
#define TEST_SUITE_CONTENTS TEST_CASES(RUN_CASE, COMMA)
TEST_SUITE(mainSuite, TEST_SUITE_CONTENTS);
#undef RUN_CASE
#undef TEST_SUITE_CONTENTS

int main(int argc, const char **argv) {
    UNUSED(argc);
    UNUSED(argv);
    return kft_run_test_suite(mainSuite, mainSuiteLength, NULL);
}
